////////////////////////////////////////////////////////////////////////////////
//                         MAX114 4-Channel ADC
//                               Example
//
// Filename     : 18F2550 MAX114.c
// Programmer   : Steven Cholewiak, www.semifluid.com
// Version      : Version 1.0 - 05/01/2006
// Remarks      : This example compares the read speed for the MAX114 to the
//                internal ADC of the PIC. It is important to use the same
//                analog source for all five analog inputs to gauge the
//                accuracy of each.
//
//                More information on the circuit can be found at:
//                http://www.semifluid.com/PIC18F2550_MAX114.html
////////////////////////////////////////////////////////////////////////////////

/*
PIC18F2550 Pinout

                          --------------
            MCLR Pullup -|1           28|-----------------------
          Analog Source -|2           27|---------------------- |
     --------------------|3           26|--------------------- ||
    | -------------------|4           25|-------------------- |||
    || ------------------|5           24|------------------- ||||
    ||| -----------------|6           23|------------------ |||||
    ||||                -|7           22|----------------- ||||||
    ||||         Ground -|8           21|---------------- |||||||
    ||||           OSC1 -|9           20|- +5V           ||||||||
    ||||           OSC2 -|10          19|- Ground        ||||||||
    ||||                -|11          18|- RS232 RX      ||||||||
    ||||                -|12          17|- RS232 TX      ||||||||
    ||||                -|13          16|-               ||||||||
    ||||                -|14          15|-               ||||||||
    ||||                  --------------                 ||||||||
    ||||                                                 ||||||||
    ||| -----------------------------------------        ||||||||
    || ----------------------------------------- |       ||||||||
    ||                                          ||       ||||||||
    ||             -----------------------------++------- |||||||
    ||            +-----------------------------++-------- ||||||
    ||           +------------------------------++--------- |||||
    ||          +-------------------------------++---------- ||||
    ||         ||||                             ||           ||||
    ||         ||||       --------------        ||           ||||
    ||         ||||     -|1  IN4  VDD 24|- +5V  ||           ||||
    ||         ||||     -|2  IN3  PWD 23|- +5V  ||           ||||
    ||         ||||     -|3  IN2   A0 22|------- |           ||||
    ||         ||||     -|4  IN1   A1 21|--------            ||||
    ||         |||| +5V -|5  MDE   D7 20|--------------------+++
    ||         ||| ------|6  D0    D6 19|--------------------++
    ||         || -------|7  D1    D5 18|--------------------+
    ||         | --------|8  D2    D4 17|--------------------
    ||          ---------|9  D3    CS 16|--------
     +----------.--------|10 RD    WR 15|------- |
     |          |       -|11 INT  RF+ 14|- +5V  ||
     |          |   GND -|12 GND  RF- 13|- GND  ||
     |          |         --------------        ||
     |          |                               ||
     |           ------------------------------- |
      -------------------------------------------
*/

#include <18F2550.h>
#device ADC=8
#fuses HSPLL,NOWDT,NOPROTECT,NOLVP,NODEBUG,USBDIV,PLL5,CPUDIV1,VREGEN
#use delay(clock=48000000)

#build(reset=0x1, interrupt=0x8)          // Necessary for Bootloader
#ORG 0x0F00,0x0FFF {}                     // Necessary for Bootloader

#use rs232(stream=PC, baud=115200, xmit=PIN_C6, rcv=PIN_C7, ERRORS)
//#define MAX114_MODE            //Pulled High
#define MAX114_RD      PIN_A1
//#define MAX114_INT             //Ignored
//#define MAX114_WR              //Connected to MAX114_RD
#define MAX114_CS      PIN_A2
#define MAX114_A0      PIN_A3
#define MAX114_A1      PIN_A4
//#define MAX114_PWRDN           //Pulled High

#INT_RDA
void serial_isr()                         // Serial Interrupt
{
   if (fgetc(PC) == 0x12 & fgetc(PC) == 0x34 & fgetc(PC) == 0x56 & fgetc(PC) == 0x78 & fgetc(PC) == 0x90) #asm reset #endasm
}

void main() {
   int8 theCount, tempVolt;
   int16 period, i;
   float tempFloat;
   int8 voltageMeasure[512];

   SETUP_ADC_PORTS(AN0);
   SETUP_ADC(ADC_CLOCK_DIV_64);
   SETUP_TIMER_0(RTCC_INTERNAL|RTCC_DIV_1);
   SETUP_TIMER_1(T1_DISABLED);
   SETUP_TIMER_2(T2_DISABLED, 127, 1);
   SETUP_TIMER_3(T3_INTERNAL | T3_DIV_BY_1);
   SETUP_CCP1(CCP_OFF);
   SETUP_CCP2(CCP_OFF);

   enable_interrupts(INT_RDA);
   enable_interrupts(GLOBAL);

   set_adc_channel(0);
   delay_ms(1);

   OUTPUT_HIGH(MAX114_CS);
   OUTPUT_HIGH(MAX114_RD);

   i = 0;

   delay_ms(500);
   printf("OK\n\r");
   delay_ms(200);

   while (TRUE) {
      set_timer3(0);
      OUTPUT_HIGH(MAX114_A0);    // Set for CH1 & Read CH0
      OUTPUT_LOW(MAX114_A1);
      OUTPUT_LOW(MAX114_CS);
      OUTPUT_LOW(MAX114_RD);
      tempVolt = INPUT_B();
      OUTPUT_HIGH(MAX114_RD);
      OUTPUT_HIGH(MAX114_CS);
      voltageMeasure[i] = tempVolt;
      period = get_timer3();   // 1000/((clock/4)/1) for ms

      tempFloat = (float) 5/255;
      tempFloat = tempVolt * tempFloat;
      printf("%lu\n\rMAX114 CH0: %u, %f, %lu\n\r", i, tempVolt, tempFloat, period);

      set_timer3(0);
      OUTPUT_LOW(MAX114_A0);     // Set for CH2 & Read CH1
      OUTPUT_HIGH(MAX114_A1);
      OUTPUT_LOW(MAX114_CS);
      OUTPUT_LOW(MAX114_RD);
      tempVolt = INPUT_B();
      OUTPUT_HIGH(MAX114_RD);
      OUTPUT_HIGH(MAX114_CS);
      voltageMeasure[i] = tempVolt;
      period = get_timer3();   // 1000/((clock/4)/1) for ms

      tempFloat = (float) 5/255;
      tempFloat = tempVolt * tempFloat;
      printf("MAX114 CH1: %u, %f, %lu\n\r", tempVolt, tempFloat, period);

      set_timer3(0);
      OUTPUT_HIGH(MAX114_A0);    // Set for CH3 & Read CH2
      OUTPUT_HIGH(MAX114_A1);
      OUTPUT_LOW(MAX114_CS);
      OUTPUT_LOW(MAX114_RD);
      tempVolt = INPUT_B();
      OUTPUT_HIGH(MAX114_RD);
      OUTPUT_HIGH(MAX114_CS);
      voltageMeasure[i] = tempVolt;
      period = get_timer3();   // 1000/((clock/4)/1) for ms

      tempFloat = (float) 5/255;
      tempFloat = tempVolt * tempFloat;
      printf("MAX114 CH2: %u, %f, %lu\n\r", tempVolt, tempFloat, period);

      set_timer3(0);
      OUTPUT_LOW(MAX114_A0);     // Set for CH0 & Read CH3
      OUTPUT_LOW(MAX114_A1);
      OUTPUT_LOW(MAX114_CS);
      OUTPUT_LOW(MAX114_RD);
      tempVolt = INPUT_B();
      OUTPUT_HIGH(MAX114_RD);
      OUTPUT_HIGH(MAX114_CS);
      voltageMeasure[i] = tempVolt;
      period = get_timer3();   // 1000/((clock/4)/1) for ms

      tempFloat = (float) 5/255;
      tempFloat = tempVolt * tempFloat;
      printf("MAX114 CH3: %u, %f, %lu\n\r", tempVolt, tempFloat, period);

      set_timer3(0);
      tempVolt = READ_ADC();
      voltageMeasure[i] = tempVolt;
      period = get_timer3();   // 1000/((clock/4)/1) for ms

      tempFloat = (float) 5/255;
      tempFloat = tempVolt * tempFloat;
      printf("ADC: %u, %f, %lu\n\r\n\r", tempVolt, tempFloat, period);

      i++;
      if (i >= 512) i=0;

      delay_ms(1000);
   }
}
